import { defineComponent, ref } from "vue";
import PureTable from "@pureadmin/table";
import { pageContentProps } from "./types";
import { TableProBar } from "../../ReTable";
import { createActionProps, createOperation } from "./utils/create-operation";
import { useRequeset } from "./hooks/use-request";
import { Operation } from "./actions";
import "./styles/index.scss";
import { onBeforeRouteLeave } from "vue-router";
import { useAppStoreHook } from "/@/store/modules/app";

const PageConent = defineComponent({
  name: "PageContent",
  props: pageContentProps,
  emits: [
    "createBtnClick",
    "updateBtnClick",
    "previewBtnClick",
    "statusBtnClick",
    "selectionChange"
  ],
  setup(props, { expose, emit, slots }) {
    //根据传入的api决定表格需要的操作
    const action = createOperation(props.contentConfig.apis);

    const tableRef = ref<InstanceType<typeof PureTable>>(null);

    const { setSearchParams } = useAppStoreHook();

    const {
      loading,
      dataList,
      onSearch,
      getPageData,
      getBasicParams,
      getPagination,
      onBeforeAction,
      paginationConfig
    } = useRequeset(props);

    function handleSelection(value) {
      emit("selectionChange", value);
    }

    /**
     * 渲染需要的操作按钮
     */
    function renderOperation(row) {
      const { contentConfig } = props;
      const {
        apis,
        inv,
        notPreview,
        powers,
        payProps,
        returnProps,
        delTooltip,
        showPreview,
        showDelete,
        isRequesetArray
      } = contentConfig;

      const view = showPreview ? showPreview(row) : true;

      const del = showDelete ? showDelete(row) : true;

      return (
        <>
          {!notPreview && view && powers.some(i => i == "007") && (
            <Operation.Preview
              isInv={inv}
              onPreview={() => emit("previewBtnClick", row)}
            />
          )}

          {slots.custom && slots.custom(row)}

          {returnProps && (
            <Operation.Tag
              row={row}
              tagProps={returnProps}
              onReload={() => onSearch()}
            />
          )}

          {payProps && (
            <Operation.Tag
              row={row}
              tagProps={payProps}
              isPay
              onReload={() => onSearch()}
            />
          )}

          {action.update && powers.some(i => i == "005") && (
            <Operation.Update
              onUpdate={() => emit("updateBtnClick", row)}
              {...createActionProps("update", apis)}
            />
          )}

          {action.status &&
            ((powers.some(i => i == "003") && row.status + "" === "0") ||
              (powers.some(i => i == "004") && row.status + "" === "1")) && (
              <Operation.Status
                row={row}
                prop={contentConfig.statusProp}
                onReload={() => onSearch()}
              isRequesetArray={isRequesetArray}
                {...createActionProps("status", apis)}
              />
            )}

          {del && action.delete && powers.some(i => i == "006") && (
            <Operation.Delete
              row={row}
              prop={contentConfig.deleteProp}
              delTooltip={delTooltip}
              onReload={() => onSearch()}
              {...createActionProps("delete", apis)}
            />
          )}
        </>
      );
    }

    onBeforeRouteLeave(({ path }) => {
      //到详情页保存查询参数 到其他页面清空查询参数
      const isDetail = path.indexOf('Detail') >= 0;
      const basicParams = isDetail ? getBasicParams() : null;
      const pagination = isDetail ? getPagination() : null;
      setSearchParams(basicParams, pagination);
    })

    function renderExpand(row) {
      const { contentConfig } = props;
      const { showExpand } = contentConfig;
      if (!showExpand || !showExpand(row)) return null;
      return slots.expand && slots.expand(row);
    }

    function renderPureTable(size, checkList) {
      const { contentConfig } = props;
      const { columns, treeProps, rowKey, isTree } = contentConfig;

      return (
        <PureTable
          ref={isTree && tableRef}
          border
          defaultExp
          align="left"
          showOverflowTooltip
          defaultExpandAll
          table-layout="auto"
          size={size}
          rowKey={rowKey}
          columns={columns}
          treeProps={treeProps}
          data={dataList.value}
          checkList={checkList}
          paginationSmall={size === "small" ? true : false}
          headerCellStyle={{ background: "#fafafa", color: "#606266" }}
          onSelectionChange={handleSelection}
          {...paginationConfig}
          v-slots={{
            operation: ({ row }) => renderOperation(row),
            expand: ({ row }) => renderExpand(row)
          }}
        />
      );
    }

    expose({
      onSearch,
      getPageData,
      onBeforeAction
    });

    return () => {
      const { contentConfig } = props;
      const { title, powers } = contentConfig;

      return (
        <TableProBar
          tableRef={tableRef.value?.getTableRef()}
          title={title}
          dataList={dataList.value}
          onRefresh={onSearch}
          loading={loading.value}
          v-slots={{
            buttons: () =>
              (slots.create && slots.create()) ||
              (action.create && powers.some(i => i == "002") && (
                <Operation.Create
                  {...createActionProps("create", contentConfig.apis)}
                  onCreate={() => emit("createBtnClick")}
                />
              )),
            default: ({ size, checkList }) => renderPureTable(size, checkList)
          }}
        />
      );
    };
  }
});

export default PageConent;
